@@ -1,7 +1,7 @@
 /********************************************************************
  * $Author: jgoerzen $
- * $Revision: 1.3 $
- * $Date: 2001/01/17 21:48:05 $
+ * $Revision: 1.5 $
+ * $Date: 2002/03/19 20:22:42 $
  * $Source: /var/cvs/gopher/gopher/pager.c,v $
  * $State: Exp $
  *
@@ -15,6 +15,13 @@
  *********************************************************************
  * Revision History:
  * $Log: pager.c,v $
+ * Revision 1.5  2002/03/19 20:22:42  jgoerzen
+ * Logged regex changes.
+ * Include STRstring.h and strcasestr proto.
+ *
+ * Revision 1.4  2002/02/12 21:20:14  jgoerzen
+ * Made files using strcasecmp() include util.h
+ *
  * Revision 1.3  2001/01/17 21:48:05  jgoerzen
  * Many fixes and tune-ups.  Now compiles cleanly with -Wall -Werror!
  *
@@ -88,9 +95,11 @@
 
 #include "gopher.h"
 #include "CURcurses.h"
+#include "STRstring.h"
 #include "fileio.h"
 #include "Malloc.h"
 #include "pager.h"
+#include "util.h"
 
 #ifdef VMS
 #include <stat.h>
