@@ -1,8 +1,8 @@
 /********************************************************************
- * $Author: lindner $
- * $Revision: 3.52 $
- * $Date: 1995/11/03 18:13:58 $
- * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopherd/ftp.c,v $
+ * $Author: jgoerzen $
+ * $Revision: 1.5 $
+ * $Date: 2002/02/12 21:20:14 $
+ * $Source: /var/cvs/gopher/gopherd/ftp.c,v $
  * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
@@ -15,6 +15,23 @@
  *********************************************************************
  * Revision History:
  * $Log: ftp.c,v $
+ * Revision 1.5  2002/02/12 21:20:14  jgoerzen
+ * Made files using strcasecmp() include util.h
+ *
+ * Revision 1.4  2001/01/17 19:30:25  jgoerzen
+ * Change many sprintf -> snprintf
+ *
+ * Revision 1.3  2000/12/21 05:33:20  s2mdalle
+ *
+ *
+ * Miscellaneous code cleanups
+ *
+ * Revision 1.2  2000/12/20 01:19:20  jgoerzen
+ * Added patches from David Allen <s2mdalle@titan.vcu.edu>
+ *
+ * Revision 1.1.1.1  2000/08/19 00:28:56  jgoerzen
+ * Import from UMN Gopher 2.3.1 after GPLization
+ *
  * Revision 3.52  1995/11/03  18:13:58  lindner
  * Coen: ftp gw fixes..
  *
@@ -194,7 +211,6 @@
  *
  *********************************************************************/
 
-
 /* -------------------------------------------------
  *     g2fd.c          Gopher to FTP gateway daemon.
  *     Version 0.3 Hacked up: April 1992.  Farhad Anklesaria.
@@ -211,6 +227,7 @@
 #include "gopherd.h"
 #include "ftp.h"
 #include "url.h"
+#include "util.h"
 #include <signal.h>
 
 #include <stdio.h>
@@ -221,8 +238,6 @@
 
 #include "Debug.h"
 
-
-
 /** Global socket value **/
 static int Gsockfd = -1;  
 
@@ -295,11 +310,12 @@ FTPconnect(FTP *ftp, char *host)
 {
      int newcontrol;
      char message[256];
-     
+     void (*fptr_FTPcleanup)() = FTPcleanup;
+
      /** Get ready for some cleanup action **/
-     signal(SIGPIPE, FTPcleanup);
-     signal(SIGINT, FTPcleanup);
-     signal(SIGALRM, FTPcleanup);
+     signal(SIGPIPE, fptr_FTPcleanup);
+     signal(SIGINT, fptr_FTPcleanup);
+     signal(SIGALRM, fptr_FTPcleanup);
      
      newcontrol = SOCKconnect(host,21);
 
@@ -308,8 +324,9 @@ FTPconnect(FTP *ftp, char *host)
 			 GSsendHeader(Gsockfd, -1);
 			 SentGPHeader = TRUE;
 		 }
-		 sprintf (message, "failed to connect to %s %s\n",
-				  host, sys_err_str());
+		 snprintf(message, sizeof(message), 
+			  "failed to connect to %s %s\n",
+			  host, sys_err_str());
 		 FTPerrorMessage(ftp, message);
 		 return(FALSE);
 	 }
@@ -530,7 +547,7 @@ FTPcommand(FTP  *ftp, char *command, char *parameter, int  errorlevel)
      char commandline[512];
      char message[256];
      
-     sprintf(commandline, command, parameter);
+     snprintf(commandline, sizeof(commandline), command, parameter);
      if (FTPsend(ftp, commandline))
 	  return(-1); /** Error condition **/
 
@@ -579,7 +596,8 @@ FTPopenData(FTP *ftp, char *command, char *file, int errorlevel)
      if ((ftp_dataport = SOCKlisten(&we)) < 0)
 	  return;
      
-     sprintf(theline, "PORT %d,%d,%d,%d,%d,%d",
+     snprintf(theline, sizeof(theline),
+	      "PORT %d,%d,%d,%d,%d,%d",
 	     (htonl(we.sin_addr.s_addr) >> 24) & 0xFF,
 	     (htonl(we.sin_addr.s_addr) >> 16) & 0xFF,
 	     (htonl(we.sin_addr.s_addr) >>  8) & 0xFF,
@@ -794,7 +812,8 @@ FTPwalkchdir(FTP  *ftp, char *path)
 			 *cp = '?';
 
  	  if (!notascii && strchr(vmspath, ' ')) {
-	       sprintf (tmppath, "\"%s\"", vmspath);
+	       snprintf(tmppath, sizeof(tmppath),
+			"\"%s\"", vmspath);
 	       FTPchdir(ftp, tmppath);
  	  } else
 	       FTPchdir(ftp, vmspath);
@@ -834,7 +853,7 @@ FTPerrorMessage(FTP *ftp, char *message)
 
      ZapCRLF(message);
 
-     sprintf(errmsg, "3%s\t\terror.host\t1\r\n", message);
+     snprintf(errmsg, sizeof(errmsg), "3%s\t\terror.host\t1\r\n", message);
      if (GFTPplus)
 	  writerr = writestring(Gsockfd, "+INFO: ");
      if (writerr || writestring(Gsockfd, errmsg)) {
@@ -863,9 +882,11 @@ FTPinfoMessage(FTP *ftp, char *message)
 
      if (GETDIR) {
 	  if ((int)strlen(message) > 3 && message[3] == '-')
-	       sprintf(errmsg, "i%s\t\terror.host\t1\r\n", &message[4]);
+	       snprintf(errmsg, sizeof(errmsg),
+			"i%s\t\terror.host\t1\r\n", &message[4]);
 	  else
-	       sprintf(errmsg, "i%s\t\terror.host\t1\r\n", message);
+	       snprintf(errmsg, sizeof(errmsg),
+			"i%s\t\terror.host\t1\r\n", message);
 	  
 	  if (GFTPplus)
 		 writerr = writestring(Gsockfd, "+INFO: ");
@@ -963,7 +984,8 @@ GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 
 
      if (*ftppass == '\0') {
-	  sprintf(ftppass, "-gopher@%s", Zehostname);
+	  snprintf(ftppass, sizeof(ftppass), 
+		   "-gopher@%s", Zehostname);
      }
      if (Gopherstow == NULL)
 	  Gopherstow = GSnew();
@@ -985,9 +1007,11 @@ GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 	  GSsetHost(gs, Zehostname);
 	  GSsetPort(gs, GopherPort);
 	  if (strcmp(ftpuser, "anonymous") != 0) {
- 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), ftppath);
+ 	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s%s", URLget(ftpurl), ftppath);
 	  } else {
-	       sprintf(tmpstr, "ftp:%s@%s", ftphost, ftppath);
+	       snprintf(tmpstr, sizeof(tmpstr),
+			"ftp:%s@%s", ftphost, ftppath);
 	  }
 	  GSsetPath(gs, tmpstr);
 	  GSsetTitle(gs, tmpstr);
@@ -1049,7 +1073,8 @@ GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 			 if (!isascii(*cp))
 			      *cp = '?';
 		    if (strchr(ftppath, ' ')) {
-			 sprintf (ftppass, "\"%s\"", ftppath);
+			 snprintf (ftppass, sizeof(ftppass), 
+				   "\"%s\"", ftppath);
 			 FTPchdir(ftp, ftppass);
 		    } else
 			 FTPchdir(ftp, ftppath);
@@ -1135,7 +1160,7 @@ GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 		    if (!isascii(*cp))
 			 *cp = '?';
 	       if (strchr(ftppath, ' ')) {
-		    sprintf (ftppass, "\"%s\"", ftppath);
+		    snprintf (ftppass, sizeof(ftppass), "\"%s\"", ftppath);
 		    fname = ftppass;
 	       } else
 		    fname = ftppath;
@@ -1256,17 +1281,21 @@ GopherList(FTP *ftp, char *bufptr, char *theName, GopherObj *gs,
      return(Gzerotype);
 }
 
-
+/* NOTE: 
+ * arg 8 has been changed from char **ftppass to char *ftppass given its
+ * similarity to ParseVMSList and ParseOS2List.  But what gives?  It isn't
+ * even used in the function body.  
+ */
 int
 ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
-	      int cols, GopherObj *gs, char *ftpuser, char **ftppass)
+	      int cols, GopherObj *gs, char *ftpuser, char *ftppass)
 {
-     int i, end;
-     int gap, objsize;
+     int i=0, end=0;
+     int gap=0, objsize=0;
      char *dirname, *alias, *group=NULL, *size=NULL, *month=NULL, 
           *novellsize=NULL;
      char tmpstr[64], sizestr[16], datestr[32];
-     char *cp1;
+     char *cp1=NULL;
 
      end = strlen(bufptr);
      while (isspace(bufptr[end-1]))
@@ -1340,7 +1369,7 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	  return (-1);
 
      if (GFTPplus) {
-	  sprintf(tmpstr, "%s <%s>", 
+	  snprintf(tmpstr, sizeof(tmpstr), "%s <%s>", 
 		  GDCgetAdmin(Config), GDCgetAdminEmail(Config));
 	  GSsetAdmin(gs, tmpstr);
      }
@@ -1362,6 +1391,18 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	  month = group;
 	  size = novellsize;
 	  break;
+          /* Extra unused cases added to clarify assumptions and so the 
+           * compiler won't complain.
+           */
+     case FTP_UNKNOWN:
+     case FTP_VMS:
+     case FTP_UNIX:
+     case FTP_MTS:
+     case FTP_WINNT:
+     case FTP_VM:
+     case FTP_UNIX_L8:
+     case FTP_MICRO_VAX:
+     case FTP_OS2:
      }
 
  try_size_again:
@@ -1370,9 +1411,10 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	      isalpha(*(month+2)) && isspace(*(month+3))) {
 	       objsize = (int) strtol(size, (char **)NULL, 10);
 	       if (objsize >= 1024)
-		    sprintf(sizestr, "%dk", objsize/1024);
+		    snprintf(sizestr, sizeof(sizestr), "%dk", objsize/1024);
 	       else
-		    sprintf(sizestr, ".%dk", (objsize*10)/1024);
+		    snprintf(sizestr, sizeof(sizestr), 
+			     ".%dk", (objsize*10)/1024);
 	       strncpy (datestr, month, alias-month-1);
 	       datestr[alias-month-1] = '\0';
 	  } else {
@@ -1432,13 +1474,15 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	       sprintf(bufptr, "%s/", IntName);
 	       
 	       GSsetType(gs, A_DIRECTORY);
-	       sprintf (tmpstr, "%s [%s]", IntName, datestr);
+	       snprintf (tmpstr, sizeof(tmpstr), "%s [%s]", IntName, datestr);
 	       GSsetTitle(gs, tmpstr);
 
 	       if (strcmp(ftpuser, "anonymous") != 0)
- 		    sprintf(tmpstr, "%s%s/", URLget(ftpurl), IntName);
+ 		    snprintf(tmpstr, sizeof(tmpstr),
+			     "%s%s/", URLget(ftpurl), IntName);
 	       else
-		    sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
+		    snprintf (tmpstr, sizeof(tmpstr), 
+			      "ftp:%s@%s/%s/", GSgetHost(gs),
 			     GSgetPath(gs), IntName);
 
 	       GSsetPath(gs, tmpstr);
@@ -1452,17 +1496,20 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 
 	       i = GopherType(ftp, dirname+gap, theName);
 	       strcpy(theName+1, IntName); 
-	       sprintf(tmpstr, "%s@", dirname+gap);
-	       strcat(theName+1, tmpstr);
+	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s@%s", dirname+gap, theName + 1);
 	       strcpy(bufptr, IntName);
 	       GSsetType(gs, theName[0]);
-	       sprintf (tmpstr, "%s (%s) [%s]", IntName, sizestr, datestr);
+	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s (%s) [%s]", IntName, sizestr, datestr);
 	       GSsetTitle(gs, tmpstr);
 	       if (strcmp(ftpuser, "anonymous") != 0)
- 		    sprintf(tmpstr, "%s%s",URLget(ftpurl), theName+1);
+ 		    snprintf(tmpstr, sizeof(tmpstr),
+			     "%s%s",URLget(ftpurl), theName+1);
 	       else
-		    sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-			    GSgetPath(gs), theName+1);
+		    snprintf(tmpstr, sizeof(tmpstr),
+			     "ftp:%s@%s/%s", GSgetHost(gs),
+			     GSgetPath(gs), theName+1);
 
 	       GSsetPath(gs, tmpstr);
 	       if (GFTPplus) {
@@ -1488,18 +1535,20 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	  sprintf(bufptr, "%s/", IntName);
 
 	  GSsetType(gs, A_DIRECTORY);
-	  sprintf (tmpstr, "%s [%s]", IntName, datestr);
+	  snprintf (tmpstr, sizeof(tmpstr), "%s [%s]", IntName, datestr);
 	  GSsetTitle(gs, tmpstr);
- 	  if (cp1 = strchr(IntName, '/'))
+ 	  if ((cp1 = strchr(IntName, '/')))
 	       for ( ; *cp1; cp1++ ) {
 		    if (*cp1 == '/')
 			 *cp1 = '*';
 	       }
 	  
 	  if (strcmp(ftpuser, "anonymous") != 0) 
- 	       sprintf (tmpstr, "%s%s/", URLget(ftpurl), IntName);
+ 	       snprintf (tmpstr, sizeof(tmpstr),
+			 "%s%s/", URLget(ftpurl), IntName);
 	  else
-	       sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs), GSgetPath(gs), 
+	       snprintf (tmpstr, sizeof(tmpstr),
+			 "ftp:%s@%s/%s/", GSgetHost(gs), GSgetPath(gs), 
 			IntName);
 	  GSsetPath(gs, tmpstr);
 	  if (GFTPplus) {
@@ -1516,12 +1565,15 @@ ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
 	  strcpy(bufptr, theName+1);
 	  GopherType(ftp, IntName, theName);
 	  GSsetType(gs, theName[0]);
-	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
+	  snprintf (tmpstr, sizeof(tmpstr),
+		    "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  if (strcmp(ftpuser, "anonymous") != 0)
- 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+ 	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s%s", URLget(ftpurl), theName+1);
 	  else
-	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
+	       snprintf(tmpstr, sizeof(tmpstr),
+			"ftp:%s@%s/%s", GSgetHost(gs),
 		       GSgetPath(gs), theName+1);
 
 	  GSsetPath(gs, tmpstr);
@@ -1565,14 +1617,14 @@ ParseVMSList(FTP  *ftp, char *bufptr, char *IntName, char *theName,
      GopherType(ftp, IntName, theName);
      GSsetType(gs, theName[0]);
      if (GFTPplus) {
-	  sprintf(tmpstr, "%s <%s>", 
+	  snprintf(tmpstr, sizeof(tmpstr), "%s <%s>", 
 		  GDCgetAdmin(Config), GDCgetAdminEmail(Config));
 	  GSsetAdmin(gs, tmpstr);
      }
      for ( ; bufptr[i] && isspace(bufptr[i]); i++);
      if (isdigit(bufptr[i])) {
 	  objsize = (int) strtol(&bufptr[i], (char **)NULL, 10);
-	  sprintf(sizestr, "%dk", objsize);
+	  snprintf(sizestr, sizeof(sizestr), "%dk", objsize);
      } else if (!strncmp(&bufptr[i], "%RMS-E-PRV", strlen("%RMS-E-PRV")))
 	  return (-1);
      else {
@@ -1593,25 +1645,29 @@ ParseVMSList(FTP  *ftp, char *bufptr, char *IntName, char *theName,
      switch (i = GSgetType(gs)) {
      case A_DIRECTORY:
 	  if (strcmp(ftpuser, "anonymous") != 0)
- 	       sprintf(tmpstr, "%s%s/", URLget(ftpurl), theName+1);
+ 	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s%s/", URLget(ftpurl), theName+1);
 	  else
-	       sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
+	       snprintf(tmpstr, sizeof(tmpstr), "ftp:%s@%s/%s/", GSgetHost(gs),
 		       GSgetPath(gs), theName+1);
 
 	  GSsetPath(gs, tmpstr);
-	  sprintf (tmpstr, "%s [%s]", theName+1, datestr);
+	  snprintf(tmpstr, sizeof(tmpstr), "%s [%s]", theName+1, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  return(A_DIRECTORY);
 	  break;
      default: 
 	  
 	  if (strcmp(ftpuser, "anonymous") != 0)
- 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+ 	       snprintf(tmpstr, sizeof(tmpstr),
+			"%s%s", URLget(ftpurl), theName+1);
 	  else
-	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-		       GSgetPath(gs), theName+1);
+	       snprintf(tmpstr, sizeof(tmpstr),
+			"ftp:%s@%s/%s", GSgetHost(gs),
+			GSgetPath(gs), theName+1);
 	  GSsetPath(gs, tmpstr);
-	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
+	  snprintf (tmpstr, sizeof(tmpstr),
+		    "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  
 	  break;
@@ -1634,9 +1690,9 @@ ParseOS2List(FTP *ftp, char *bufptr, char *IntName, char *theName,
      if (isdigit(bufptr[i])) {
 	  objsize = (int) strtol(&bufptr[i], (char **)NULL, 10);
 	  if (objsize >= 1024)
-	       sprintf(sizestr, "%dk", objsize/1024);
+	       snprintf(sizestr, sizeof(sizestr), "%dk", objsize/1024);
 	  else
-	       sprintf(sizestr, ".%dk", (objsize*10)/1024);
+	       snprintf(sizestr, sizeof(sizestr), ".%dk", (objsize*10)/1024);
      } else {
 	  LOGGopher(Gsockfd, "Couldn't parse %s", bufptr);
 	  return (-1);
@@ -1671,8 +1727,8 @@ ParseOS2List(FTP *ftp, char *bufptr, char *IntName, char *theName,
      if (GFTPplus) {
 	  GSsetModDate(gs, datestr);
 	  AddDefaultView(gs, objsize*1024, NULL);
-	  sprintf(tmpstr, "%s <%s>", 
-		  GDCgetAdmin(Config), GDCgetAdminEmail(Config));
+	  snprintf(tmpstr, sizeof(tmpstr), "%s <%s>", 
+		   GDCgetAdmin(Config), GDCgetAdminEmail(Config));
 	  GSsetAdmin(gs, tmpstr);
      }
      GopherType(ftp, &bufptr[i], theName);
@@ -1680,13 +1736,14 @@ ParseOS2List(FTP *ftp, char *bufptr, char *IntName, char *theName,
      switch (i = GSgetType(gs)) {
      case A_DIRECTORY:
 	  if (strcmp(ftpuser, "anonymous") != 0)
- 	       sprintf(tmpstr, "%s%s/", URLget(ftpurl), theName+1);
+ 	       snprintf(tmpstr, sizeof(tmpstr), 
+			"%s%s/", URLget(ftpurl), theName+1);
 	  else
-	       sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
-		       GSgetPath(gs), theName+1);
+	       snprintf(tmpstr, sizeof(tmpstr), "ftp:%s@%s/%s/", GSgetHost(gs),
+			GSgetPath(gs), theName+1);
 
 	  GSsetPath(gs, tmpstr);
-	  sprintf (tmpstr, "%s [%s]", theName+1, datestr);
+	  snprintf (tmpstr, sizeof(tmpstr), "%s [%s]", theName+1, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  return(A_DIRECTORY);
 	  break;
@@ -1694,12 +1751,14 @@ ParseOS2List(FTP *ftp, char *bufptr, char *IntName, char *theName,
      default: 
 	  i = GSsetType(gs, theName[0]);
 	  if (strcmp(ftpuser, "anonymous") != 0)
- 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+ 	       snprintf(tmpstr, sizeof(tmpstr), 
+			"%s%s", URLget(ftpurl), theName+1);
 	  else
-	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-		       GSgetPath(gs), theName+1);
+	       snprintf(tmpstr, sizeof(tmpstr), "ftp:%s@%s/%s", GSgetHost(gs),
+			GSgetPath(gs), theName+1);
 	  GSsetPath(gs, tmpstr);
-	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
+	  snprintf (tmpstr, sizeof(tmpstr),
+		    "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  break;
      }
